==> MKLpy-0.4.2.tar.gz: MKLpy-0.4.2/MKLpy/arrange.py <==
"""
.. codeauthor:: Ivano Lauriola <ivanolauriola@gmail.com>

===============
Arrange Kernels
===============

.. currentmodule:: MKLpy.arrange

This module contains functions able to combine a list of kernels in a single one.

"""

import numpy as np
import types


def summation (K_list, weights = None):
    """perform the weighted summation of each kernel in K_list.

    Parametes
    ---------
    K_list : (l,n,m) ndarray, kernel_list or array_like,
             where *l* is the number of kernels in list and *(n,m)* is the shape of kernels.
    weights : (l) array_like,
              if it is not defined, then is performed the simple summation.

    Returns
    -------
    K : (n,m) ndarray,
        the summation of *l* kernels in list.

    Examples
    --------
    >>> Klist   = np.array([HPK_kernel(X,degree=i) for i in range(1,11)])
    >>> weights = range(1,11)
    >>> Ksum    = summation(Klist)
    >>> Kwsum   = summation(Klist,weights)
    """
        
    K = np.zeros(K_list[0].shape, dtype = np.double)
    l = len(K_list)
    weights = np.ones(l, dtype = np.double) if weights is None else weights
    for w,ker in zip(weights, K_list):
        K = K + w * ker
    return K


def multiplication (K_list, weights = None):
    """perform the weighted multiplication of each kernel in K_list.

    Parametes
    ---------
    K_list : (l,n,m) ndarray or kernel_list,
             where *l* is the number of kernels in list and *(n,m)* is the shape of kernels.
    weights : (l) array_like,
              if it is not defined, then is performed the simple multiplication.

    Returns
    -------
    K : (n,m) ndarray,
        the multiplication of *l* kernels in list.

    Examples
    --------
    >>> Klist   = np.array([HPK_kernel(X,degree=i) for i in range(1,11)])
    >>> weights = range(1,11)
    >>> Ksum    = multiplication(Klist)
    >>> Kwsum   = multiplication(Klist,weights)
    """
    K = np.ones(K_list[0].shape, dtype = np.float64)
    l = len(K_list)
    weights = np.ones(l, dtype = np.double) if type(weights) == types.NoneType else weights
    for w,ker in zip(weights, K_list):
        K = K * w * ker
    return K


def average (K_list, weights = None):
    """perform the mean of kernels in K_list.

    Parametes
    ---------
    K_list : (l,n,m) ndarray or kernel_list,
             where *l* is the number of kernels in list and *(n,m)* is the shape of kernels.
    weights : (l) array_like
             if it is not defined, then is performed the simple mean.

    Returns
    -------
    K : (n,m) ndarray,
        the mean of *l* kernels in list.

    Examples
    --------
    >>> Klist   = np.array([HPK_kernel(X,degree=i) for i in range(1,11)])
    >>> weights = range(1,11)
    >>> Ksum    = mean(Klist)
    >>> Kwsum   = mean(Klist,weights)
    """
    l = len(K_list)
    weights = np.ones(l, dtype = np.double) if weights is None else weights
    K = summation(K_list, weights)
    K /= np.sum(weights)
    return K

==> MKLpy-0.6.tar.gz: MKLpy-0.6/MKLpy/arrange.py <==
# -*- coding: latin-1 -*-

"""
@author: Ivano Lauriola
@email: ivano.lauriola@phd.unipd.it, ivanolauriola@gmail.com

This file is part of MKLpy: a scikit-compliant framework for Multiple Kernel Learning
This file is distributed with the GNU General Public License v3 <http://www.gnu.org/licenses/>.  

"""

import torch
from .utils.validation import check_X


def summation (KL, weights = None):
    """perform the weighted summation of each kernel in K_list.

    Parametes
    ---------
    K_list : (l,n,m) ndarray, kernel_list or array_like,
             where *l* is the number of kernels in list and *(n,m)* is the shape of kernels.
    weights : (l) array_like,
              if it is not defined, then is performed the simple summation.

    Returns
    -------
    K : (n,m) ndarray,
        the summation of *l* kernels in list.

    Examples
    --------
    >>> Klist   = np.array([HPK_kernel(X,degree=i) for i in range(1,11)])
    >>> weights = range(1,11)
    >>> Ksum    = summation(Klist)
    >>> Kwsum   = summation(Klist,weights)
    """
    
    l = len(KL)
    weights = torch.ones(l) if weights is None else weights    
    if l != len(weights):
        raise ValueError('The weights vector and the kernels list are not aligned')

    K = check_X(KL[0]) * weights[0]
    for i in range(1,l):
        K = K + weights[i] * KL[i]
    return K


def multiplication (KL, weights = None):
    """perform the weighted multiplication of each kernel in K_list.

    Parametes
    ---------
    K_list : (l,n,m) ndarray or kernel_list,
             where *l* is the number of kernels in list and *(n,m)* is the shape of kernels.
    weights : (l) array_like,
              if it is not defined, then is performed the simple multiplication.

    Returns
    -------
    K : (n,m) ndarray,
        the multiplication of *l* kernels in list.

    Examples
    --------
    >>> Klist   = np.array([HPK_kernel(X,degree=i) for i in range(1,11)])
    >>> weights = range(1,11)
    >>> Ksum    = multiplication(Klist)
    >>> Kwsum   = multiplication(Klist,weights)
    """
    l = len(KL)
    weights = torch.ones(l) if weights is None else weights    
    if l != len(weights):
        raise ValueError('The weights vector and the kernels list are not aligned')

    K = check_X(KL[0]) * weights[0]
    for i in range(1,l):
        K = K * (weights[i] * KL[i])
    return K


def average (KL, weights = None):
    """perform the mean of kernels in K_list.

    Parametes
    ---------
    K_list : (l,n,m) ndarray or kernel_list,
             where *l* is the number of kernels in list and *(n,m)* is the shape of kernels.
    weights : (l) array_like
             if it is not defined, then is performed the simple mean.

    Returns
    -------
    K : (n,m) ndarray,
        the mean of *l* kernels in list.

    Examples
    --------
    >>> Klist   = np.array([HPK_kernel(X,degree=i) for i in range(1,11)])
    >>> weights = range(1,11)
    >>> Ksum    = mean(Klist)
    >>> Kwsum   = mean(Klist,weights)
    """
    l = len(KL)
    weights = torch.ones(l) if weights is None else weights
    K = summation(KL, weights) / torch.sum(weights)
    return K
